﻿

'*******************************************************************
' Program: Data Center Visio Rack Tool
' Author: Albert E Edlund
' Date:
'
' Purpose: load data for drawings
'
' root
'*******************************************************************


Imports Microsoft.Practices.EnterpriseLibrary.Common
Imports Microsoft.Practices.EnterpriseLibrary.Data
Imports Microsoft.Practices.EnterpriseLibrary.ExceptionHandling
Imports Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging

Imports System.Data
Imports System.Drawing
Imports System.Text

Imports Microsoft.Office.Interop
Imports Microsoft.Office.Interop.Visio
Imports System.Runtime.InteropServices

Imports System.ComponentModel
Imports System.Collections.Generic
Imports System.Collections.ObjectModel



Public Class clsDC_RackData

#Region " Properties and Definitions "


    Public m_db As Database
    ' in most of the classes the m_ds is temporary, in this class we use it to store
    ' the datatables that relate to the current drawing
    Public m_ds As DataSet
    Public m_dt As DataTable

    Private mlinkKey As visRackAppConnectivity.clsVisioSupport


    '********* visio snippet vb.net visXXXX ******************
    Private _drawingApplication As Microsoft.Office.Interop.Visio.Application
    Public Property drawingApplication() As Microsoft.Office.Interop.Visio.Application
        Get
            Return Me._drawingApplication
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Application)
            If (value Is Nothing) Then
                Throw New System.ArgumentNullException("ApplicationValueIsNull", _
                    "drawingApplication is Null.")
            End If
            Me._drawingApplication = value
        End Set
    End Property

    '********* visio snippet vb.net visXXXX ******************
    Private _drawingDocument As Microsoft.Office.Interop.Visio.Document
    Public Property drawingDocument() As Microsoft.Office.Interop.Visio.Document
        Get
            If _drawingDocument Is Nothing And _drawingApplication IsNot Nothing Then
                Return _drawingApplication.ActiveDocument
            Else
                Return Me._drawingDocument
            End If
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Document)
            If (value Is Nothing) Then
                Throw New System.ArgumentNullException("DocumentValueIsNull", _
                    "drawingDocument is Null.")
            End If
            Me._drawingDocument = value
        End Set
    End Property

    '********* visio snippet vb.net vis0044vb ******************
    Private _drawingPage As Microsoft.Office.Interop.Visio.Page
    Public Property drawingPage() As Microsoft.Office.Interop.Visio.Page
        Get
            Return Me._drawingPage
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Page)
            If (value Is Nothing) Then
                Throw New System.ArgumentNullException("Value", _
                    "drawingPage is Null.")
            End If
            Me._drawingPage = value
        End Set
    End Property

    ' do we want to play with datarecordsets
    Private m_blnDR As Boolean
    Public Property _BlnDR() As Boolean
        Get
            _BlnDR = m_blnDR
        End Get
        Set(ByVal value As Boolean)
            m_blnDR = value
        End Set
    End Property

    Public m_stencilpath As String = ""
    Public Property _StencilPath() As String
        Get
            Return Me.m_stencilpath
        End Get
        Set(ByVal value As String)
            Me.m_stencilpath = value
        End Set
    End Property

#End Region

#Region " Enumerations "

    Public Enum DC_RackTables
        tblAttrApplication          ' application data apply to operating system
        tblAppConnection            ' for application topology
        tblAppSolution              ' for application topology
        tblAttrAsset                ' asset data, MAY apply to component
        tblAttrCabinet              ' cabinet attributes to apply to component
        tblAttrCabinetRow           ' row data as qualifier for sublocation data
        tblAttrDiskLogical          ' logical disk to apply to component
        tblAttrDiskPhysical         ' physical disk to apply to component
        tblAttrDraw                 ' draw data to apply to  component (specific document and page required)
        tblAttrExternalStorage      ' external disk to apply to component
        tblAttrGeoData              ' sql geo data to apply to sublocation data
        tblAttrLocation             ' sublocation data
        tblAttrNetwork              ' network definition for port and component
        tblAttrOperatingSystem      ' operating system to apply to component
        tblAttrPortData             ' port data to apply to component
        tblAttrProcessor            ' processor data to apply to component
        tblAttrVirtualHost          ' virtual host to apply to operating system
        tblClient                   ' client table as root to location
        tblComponents               ' component data
        tblCompHierarchy            ' sql component hierarchy for component relationships
        tblLocation                 ' location data to apply to clients
        tblNetworkLink              ' network link data to apply to portdata connectivity
        tblProductData              ' root product data (used to link between components and visio)
        tblDGMasters                ' root master stencil data to apply to tables (used to apply datagraphics to attributes)
        tblTopoEdge                 ' topology edge data (mirrors network link data)
        tblTopoPoint                ' topology port data (mirrors portdata)
        tblSlotMasterData           ' slot data as part of master analysis
        tblPortMasterData           ' port data as part of master analysis

        tblPortAnalysis             ' for completeness
        tblUnknown
    End Enum

    '
    '
    '
    Public Function getTableName(ByVal intEnum As DC_RackTables) As String

        Dim strReturn As String = ""

        Dim myName As DC_RackTables = intEnum
        strReturn = myName.ToString

        Return strReturn

    End Function

    '
    '
    '
    Public Structure rackURRI
        Public clientKey As Int32
        Public locKey As Int32
        Public attrlocKey As Int32
        Public rowKey As Int32
        Public rckKey As Int32
    End Structure

    '
    '
    '
    Public Function convertURRIFormat(ByVal strURRI As String) As clsDC_RackData.rackURRI

        ' init a structure to return
        Dim urriReturn As clsDC_RackData.rackURRI = New clsDC_RackData.rackURRI
        urriReturn.clientKey = -1
        urriReturn.locKey = -1
        urriReturn.attrlocKey = -1
        urriReturn.rowKey = -1
        urriReturn.rckKey = -1
        Dim intX As Integer

        Try



            Dim arrValue As String()


            ' parse out the fullpath

            strURRI = strURRI.Replace("[", "")
            strURRI = strURRI.Replace("]", "")
            arrValue = strURRI.Split({","c}, 5, StringSplitOptions.None)


            For intX = 1 To arrValue.Count
                Select Case intX
                    Case 1
                        urriReturn.clientKey = CInt(arrValue(intX))
                    Case 2
                        urriReturn.locKey = CInt(arrValue(intX))
                    Case 3
                        urriReturn.attrlocKey = CInt(arrValue(intX))
                    Case 4
                        urriReturn.rowKey = CInt(arrValue(intX))
                    Case 5
                        urriReturn.rckKey = CInt(arrValue(intX))
                End Select

            Next intX

        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

        Return urriReturn

    End Function

    '
    '
    '
    Public Function convertURRIFormat(ByVal rckURRI As clsDC_RackData.rackURRI) As String

        Dim strReturn As String = "[]"

        Try


            Dim retSB As StringBuilder = New StringBuilder
            retSB.Append("[" & rckURRI.clientKey.ToString & ",")
            retSB.Append(rckURRI.locKey.ToString & ",")
            retSB.Append(rckURRI.attrlocKey.ToString & ",")
            retSB.Append(rckURRI.rowKey.ToString & ",")
            retSB.Append(rckURRI.rckKey.ToString & "]")

            strReturn = retSB.ToString

        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

        Return strReturn

    End Function


#End Region

#Region " Load DataTable/Visio Recordsets "

    '******************************************************
    ' base tables
    '******************************************************

#Region " ProductData "





#End Region

#Region " DGMasters "


#End Region

#Region " Attribute Assets "



#End Region

#Region " Components "


#End Region

#Region " Network Links "



#End Region


    '******************************************************
    ' location data
    '******************************************************

#Region " Clients "


#End Region

#Region " Location "



#End Region

#Region " Attribute Location "



#End Region

#Region " Attribute Cabinet Rows "


#End Region

#Region " Attribute Geography Data "


#End Region


    '******************************************************
    ' Network data
    '******************************************************


#Region " Attribute Port Data "




#End Region

#Region " Attribute Network "



#End Region

    '******************************************************
    ' Miscellaneous
    '******************************************************

#Region " Attribute Cabinets "


#End Region

#Region " Attribute Draw "


#End Region

#Region " Attribute Processor "




#End Region

    '******************************************************
    ' Disk information
    '******************************************************

#Region " Attribute (Disk) External Storage "


#End Region

#Region " Attribute Disk Logical "




#End Region

#Region " Attribute Disk Physical "




#End Region

    '******************************************************
    ' Software
    '******************************************************

#Region " Attribute Operating System "



#End Region

#Region " Attribute Applications "



    Private m_clsApps As clsAttrApplications
    Public Property _clsApps() As clsAttrApplications
        Get
            Return m_clsApps
        End Get
        Set(ByVal value As clsAttrApplications)
            Me.m_clsApps = value
        End Set
    End Property

    Private m_AppRS As Int32
    Public Property _AppRs() As Int32
        Get
            Return Me.m_AppRS
        End Get
        Set(ByVal value As Int32)
            Me.m_AppRS = value
        End Set
    End Property

    Private m_AppRecordset As Microsoft.Office.Interop.Visio.DataRecordset
    Public Property _AppRd() As Microsoft.Office.Interop.Visio.DataRecordset
        Get
            Return Me.m_AppRecordset
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.DataRecordset)
            Me.m_AppRecordset = value
            Me.m_AppRS = value.ID
        End Set
    End Property

    '
    '
    '
    Public Sub loadAppTable _
                      (Optional ByVal dtSource As System.Data.DataTable = Nothing)

        Try

            If _clsApps Is Nothing Then
                _clsApps = New clsAttrApplications(drawingApplication, drawingDocument)
            Else
                _clsApps = Nothing
                _clsApps = New clsAttrApplications(drawingApplication, drawingDocument)
            End If

            Dim dtTemp As System.Data.DataTable = New System.Data.DataTable
            _clsApps.CreateTableColumns(dtTemp)

            Dim drTemp As DataRow
            Dim strTable As String = DC_RackTables.tblAttrApplication.ToString
            dtTemp.TableName = strTable

            ' the default is to go read it from sql
            If dtSource Is Nothing Then
                _clsApps.dbSelectAllApplications()
                dtTemp = _clsApps.m_dt.Copy
            Else
                ' need to do it row by row to force our schema over
                ' what excel does
                For Each drTemp In dtSource.Rows
                    dtTemp.ImportRow(drTemp)
                Next

            End If

            If m_ds.Tables(strTable) IsNot Nothing Then
                m_ds.Tables(strTable).Clear()
                For Each drTemp In dtTemp.Rows
                    m_ds.Tables(strTable).ImportRow(drTemp)
                Next
            Else
                m_ds.Tables.Add(dtTemp.Copy)
            End If

            If _BlnDR = True Then
                Dim intCt As Integer = m_ds.Tables(strTable).Rows.Count
                If intCt = 0 Then Exit Sub
                Dim vsoRecordsets As Microsoft.Office.Interop.Visio.DataRecordsets = drawingDocument.DataRecordsets
                m_AppRecordset = vsoRecordsets.AddFromXML(mlinkKey.ConvertToAdoRecordsetXML(m_ds.Tables(strTable)), 4, strTable)
            End If


        Catch com As COMException
            '********* visio snippet vb.net vis0056vb ******************
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(com, "Com+ Policy")
            If (rethrow) Then
                Throw
            End If
        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        Finally
            ' now get rid of the class
            _clsApps = Nothing
            System.GC.Collect()
        End Try


    End Sub


#End Region

#Region " Attribute Virtual Hosts "


#End Region

#Region " Application Connections "



    Private m_clsAppConn As clsAppConnections
    Public Property _clsAppConn() As clsAppConnections
        Get
            Return m_clsAppConn
        End Get
        Set(ByVal value As clsAppConnections)
            Me.m_clsAppConn = value
        End Set
    End Property

    Private m_AppConnRS As Int32
    Public Property _AppConnRs() As Int32
        Get
            Return Me.m_AppConnRS
        End Get
        Set(ByVal value As Int32)
            Me.m_AppConnRS = value
        End Set
    End Property

    Private m_AppConnRecordset As Microsoft.Office.Interop.Visio.DataRecordset
    Public Property _AppConnRd() As Microsoft.Office.Interop.Visio.DataRecordset
        Get
            Return Me.m_AppConnRecordset
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.DataRecordset)
            Me.m_AppConnRecordset = value
            Me.m_AppConnRS = value.ID
        End Set
    End Property

    '
    '
    '
    Public Sub loadAppConnTable _
            (Optional ByVal dtSource As System.Data.DataTable = Nothing)

        Try

            If _clsAppConn Is Nothing Then
                _clsAppConn = New clsAppConnections
            Else
                _clsAppConn = Nothing
                _clsAppConn = New clsAppConnections
            End If

            Dim dtTemp As System.Data.DataTable = New System.Data.DataTable
            _clsAppConn.CreateTableColumns(dtTemp)

            Dim drTemp As DataRow
            Dim strTable As String = DC_RackTables.tblAppConnection.ToString
            dtTemp.TableName = strTable

            ' the default is to go read it from sql
            If dtSource Is Nothing Then
                _clsAppConn.dbSelectAllAppConnectionData()
                dtTemp = _clsAppConn.m_dt.Copy
            Else
                ' need to do it row by row to force our schema over
                ' what excel does
                For Each drTemp In dtSource.Rows
                    dtTemp.ImportRow(drTemp)
                Next

            End If

            If m_ds.Tables(strTable) IsNot Nothing Then
                m_ds.Tables(strTable).Clear()
                For Each drTemp In dtTemp.Rows
                    m_ds.Tables(strTable).ImportRow(drTemp)
                Next
            Else
                m_ds.Tables.Add(dtTemp.Copy)
            End If

            If _BlnDR = True Then
                Dim intCt As Integer = m_ds.Tables(strTable).Rows.Count
                If intCt = 0 Then Exit Sub

                Dim vsoRecordsets As Microsoft.Office.Interop.Visio.DataRecordsets = drawingDocument.DataRecordsets
                m_AppConnRecordset = vsoRecordsets.AddFromXML(mlinkKey.ConvertToAdoRecordsetXML(m_ds.Tables(strTable)), 4, strTable)
            End If


        Catch com As COMException
            '********* visio snippet vb.net vis0056vb ******************
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(com, "Com+ Policy")
            If (rethrow) Then
                Throw
            End If
        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        Finally
            ' now get rid of the class
            _clsAppConn = Nothing
            System.GC.Collect()
        End Try


    End Sub


#End Region

#Region " Application Solutions "



    Private m_clsAppSol As clsAppSolutions
    Public Property _clsAppSol() As clsAppSolutions
        Get
            Return m_clsAppSol
        End Get
        Set(ByVal value As clsAppSolutions)
            Me.m_clsAppSol = value
        End Set
    End Property

    Private m_AppSolRS As Int32
    Public Property _AppSolRs() As Int32
        Get
            Return Me.m_AppSolRS
        End Get
        Set(ByVal value As Int32)
            Me.m_AppSolRS = value
        End Set
    End Property

    Private m_AppSolRecordset As Microsoft.Office.Interop.Visio.DataRecordset
    Public Property _AppSolRd() As Microsoft.Office.Interop.Visio.DataRecordset
        Get
            Return Me.m_AppSolRecordset
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.DataRecordset)
            Me.m_AppSolRecordset = value
            Me.m_AppSolRS = value.ID
        End Set
    End Property

    '
    '
    '
    Public Sub loadAppSolTable _
                    (Optional ByVal dtSource As System.Data.DataTable = Nothing)

        Try


            If _clsAppSol Is Nothing Then
                _clsAppSol = New clsAppSolutions
            Else
                _clsAppSol = Nothing
                _clsAppSol = New clsAppSolutions
            End If

            Dim dtTemp As System.Data.DataTable = New System.Data.DataTable
            _clsAppSol.CreateTableColumns(dtTemp)

            Dim drTemp As DataRow
            Dim strTable As String = DC_RackTables.tblAppSolution.ToString
            dtTemp.TableName = strTable

            ' the default is to go read it from sql
            If dtSource Is Nothing Then
                _clsAppSol.dbSelectAllAppSolutionData()
                dtTemp = _clsAppSol.m_dt.Copy
            Else
                ' need to do it row by row to force our schema over
                ' what excel does
                For Each drTemp In dtSource.Rows
                    dtTemp.ImportRow(drTemp)
                Next

            End If

            ' load it locally
            If m_ds.Tables(strTable) IsNot Nothing Then
                m_ds.Tables(strTable).Clear()
                For Each drTemp In dtTemp.Rows
                    m_ds.Tables(strTable).ImportRow(drTemp)
                Next
            Else
                m_ds.Tables.Add(dtTemp.Copy)
            End If

            If _BlnDR = True Then
                Dim intCt As Integer = m_ds.Tables(strTable).Rows.Count
                If intCt = 0 Then Exit Sub

                Dim vsoRecordsets As Microsoft.Office.Interop.Visio.DataRecordsets = drawingDocument.DataRecordsets
                m_AppSolRecordset = vsoRecordsets.AddFromXML(mlinkKey.ConvertToAdoRecordsetXML(m_ds.Tables(strTable)), 4, strTable)
            End If

        Catch com As COMException
            '********* visio snippet vb.net vis0056vb ******************
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(com, "Com+ Policy")
            If (rethrow) Then
                Throw
            End If
        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        Finally
            ' now get rid of the class
            _clsAppSol = Nothing
            System.GC.Collect()
        End Try


    End Sub


#End Region


    '******************************************************
    ' 
    '******************************************************

#End Region

#Region " clear component attributes "




#End Region

#Region " Constructor "

    Public Sub New()



        Try

            mlinkKey = New visRackAppConnectivity.clsVisioSupport

            m_db = DatabaseFactory.CreateDatabase()

            If m_ds Is Nothing Then
                m_ds = New DataSet
            Else
                m_ds = Nothing
                m_ds = New DataSet
            End If

            If m_dt Is Nothing Then
                m_dt = New DataTable
            Else
                m_dt = Nothing
                m_dt = New DataTable
            End If

            _BlnDR = False
            _StencilPath = My.Settings.path_custom_stencils


        Catch com As COMException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(com, "Com+ Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try


    End Sub


#End Region


    Protected Overrides Sub Finalize()

        MyBase.Finalize()

    End Sub

    '
    '
    '
    Private Sub NullAndRelease(ByVal runtimeObject As Object)

        Try

            If Not (runtimeObject Is Nothing) Then

                ' The RCW's reference count gets incremented each time the
                ' COM pointer is passed from unmanaged to managed code.
                ' Call ReleaseComObject in a loop until it returns 0 to be
                ' sure that the underlying COM object gets released.
                Dim i As Integer
                Do
                    i = System.Runtime.InteropServices.Marshal. _
                        ReleaseComObject(runtimeObject)
                Loop While i > 0
            End If
        Finally
            runtimeObject = Nothing
        End Try
    End Sub



End Class

